from .collapsible_groupbox import (
    setup_collapsiblegroupbox,
    setup_hcollapsiblegroupbox,
    setup_vcollapsiblegroupbox,
)
from .groupbox import setup_groupbox, setup_hgroupbox, setup_vgroupbox
from .scroll_area import setup_hscrollarea, setup_scrollarea, setup_vscrollarea
from .tab_widget import setup_tabwidget

__all__ = [
    "setup_collapsiblegroupbox",
    "setup_hcollapsiblegroupbox",
    "setup_vcollapsiblegroupbox",
    "setup_groupbox",
    "setup_hgroupbox",
    "setup_vgroupbox",
    "setup_scrollarea",
    "setup_hscrollarea",
    "setup_vscrollarea",
    "setup_tabwidget",
]
